/*
 *  sqldelete. Application for removing a Host and all of it's objects from nagiosql database
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; see the file COPYING. If not, write to the
 *  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <errno.h>
#include <limits.h>
#include <string.h>
#include <mysql/mysql.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <getopt.h>
#include "common.h"

#define VERSION "1.0"
#define MAX_STRING 2048

/*
 * usage function : print the help
 *
 */

void usage( void )
{
  fprintf( stderr, "USAGE:sqldelete " );
  fprintf( stderr, " -o <hostname> -m <MYSQL server / IP> -u <MYSQL username> -p <MYSQL Password> -t <database>\n\n" );
  fprintf( stderr,
       " Options :\n"
       "  -o Single Hostname to remove\n"
       "  -f File name of multiple Host names to remove\n"
       "  -m MYSQL server / IP (Required)\n"
       "  -u MYSQL Username (Required)\n"
       "  -p MYSQL Password (Required)\n"
       "  -t MYSQL Database Name (Required)\n"
       "  -d Print Debug information\n"
       "  -r Do not restore configs on error.\n"
       "  -h -?\t\tPrint this help\n"
       "  -V \t\tPrint Version\n" );
}

/*
 * main function : -> parse command line args
 */
int main( int argc, char *argv[] )
{
  int opt;
  int exitcode = OK;

  /* Print the help if not arguments provided */
  if( argc==1 )
  {
    usage();
    return( UNKNOWN );
  }

  /*
   * get the common command line arguments
   */

  while( (opt = getopt(argc, argv, "?m:hVu:p:t:o:f:dr") ) != -1 )
  {
    switch( opt )
    {
    case '?':
    case 'h':
      /* print help */
      usage();
      exit(UNKNOWN);
      break;

    case 'V':
      print_version();
      exit( UNKNOWN );
      break;

    case 'm':
      /* MYSQL Host */
      if( strncmp( optarg, "-", 1 ) == 0 )
      {
        printf( "The MYSQL Host / IP is required for this option\n" );
        exit( UNKNOWN );
      }
      else
        mysqlhost = strdup(optarg);
      break;

    case 'u':
      /* MYSQL User Name */
      if( strncmp( optarg, "-", 1 ) == 0 )
      {
        printf( "The user name is required for this option.\n" );
        exit( UNKNOWN );
      }
      else
        username = strdup( optarg );
      break;

    case 'p':
      /* MYSQL Password */
      if( strncmp( optarg, "-", 1 ) == 0 )
      {
        printf( "The Password is required for this option\n" );
        exit( UNKNOWN );
      }
      else
        password = strdup(optarg);
      break;

    case 't':
      /* MYSQL Database Name */
      if( strncmp( optarg, "-", 1 ) == 0 )
      {
        printf( "The MYSQL Database Name is required for this option\n" );
        exit( UNKNOWN );
      }
      else
        mysqldatabase = strdup(optarg);
      break;

    case 'd':
      debug = true;
      break;

    case 'r':
      donotrestoreconfigs = true;
      break;

    case 'o':
      /* Hostname */
      singlehost = true;
      if( debug )
        printf( "The Hostname is %s\n", optarg );
      if( strncmp( optarg, "-", 1 ) == 0 )
      {
        printf( "The Hostname is required for this option\n" );
        exit( UNKNOWN );
      }
      else
      {
        insertFirsthost( strdup(optarg), &host );
      }
      break;

    case 'f':
      /* Hostname from file */
      multiplehost = true;
      if( strncmp( optarg, "-", 1 ) == 0 )
      {
        printf( "The Host file name is required for this option\n" );
        exit( UNKNOWN );
      }
      else
        multihostname = strdup( optarg );

      break;

    }
  }

  if( ( multiplehost == true ) && ( singlehost == true) )
  {
    fprintf(stderr, "Cannot use single Host option with File Host options.\n");
    exit( UNKNOWN );
  }

  //verify the configs and exit if errors
  if( verify_nagios() != 0 )
  {
    printf( "Nagios Configuration Error.\nCannot continue remove Hosts and Services\nFix the configuration errors and try again.\n");
    exit( UNKNOWN );
  }

  if( singlehost == true )
  {
    host->id = strdup( gethostname( host->hostname ) );
    if( strncmp( host->id, "0", 1 ) == 0 )
    {
      printf( "Invalid Hostname specified.\n" );
      exit( UNKNOWN );
    }
  }

  if( multiplehost == true )
  {
    openfile( multihostname );
    getidForhostList( host );
    deleteKey( &host );

    // If all of the hosts are invalid in the list, just exit
    if( host == NULL )
    {
      printf( "Invalid Hostnames in file.\n" );
      exit( UNKNOWN );
    }
    printhostList( host );
  }

  getservicename( host );
  getserviceinhostgroup( service );
  getserviceinsecondhost( service );
  gethostinhostgroup( host );
  if( debug )
  {
    printf( "Service Struct\n");
    printList( service );
    printf( "Hostgroup Struct\n");
    printList( hostgroup );
  }

  //Create a snapshot before deleting the configs. Exit if cannot create one.
  if( create_snapshot() != 0 )
    exit( UNKNOWN );

  // Do the Deletions
  deleteall( host );

  // Write the Configs
  write_configs();

  //verify the configs and revert if errors
  if( verify_nagios() != 0 )
  {
    fprintf(stderr, "Could not verify Nagios configurations.\n");

    if( donotrestoreconfigs )
    {
      system( "/usr/local/nagios/bin/nagios -v /usr/local/nagios/etc/nagios.cfg");
      exit( UNKNOWN );
    }
    else
    {
      //Revert Configs and restart nagios
      fprintf(stderr, "Reverting the configurations.\n");
      revert_configs();
      if( restart_nagios() != 0 )
        exit( UNKNOWN );
    }
  }
  else
  {
    fprintf(stderr, "Verification of the configurations passed.\n");
    //restart nagios if configs are good
    if( restart_nagios() != 0 )
      exit( OK );
  }
}

/*
 * gethostname : Gets the ID of the host_name from the MYSQL database
 *
 */
char *gethostname( char *hostname )
{
  char *status = NULL;
  char query[MAX_STRING] = {0};
  char *id = "0";

  if ((conn = mysql_init(NULL)) == NULL )
  {
    fprintf(stderr, "Could not init DB\n");
    exit( UNKNOWN );
  }

//if (mysql_real_connect(conn, "localhost", "user", "passwd", "dbname", 0, NULL, 0) == NULL)
  if( mysql_real_connect(conn, mysqlhost, username, password, mysqldatabase, 0, NULL, 0 ) == NULL )
  {
    fprintf(stderr, "DB Connection Error\n");
    exit( UNKNOWN );
  }

  snprintf( query, MAX_STRING, "select id from tbl_host WHERE host_name = (\"%s\")", hostname );
  mysqlquery(query);

  MYSQL_RES *result = mysql_store_result(conn);

  if( result == NULL )
  {
    if( debug )
      fprintf(stderr, "Host Search Result is NULL\n");
    finish_with_error(conn);
  }

  MYSQL_ROW row;

  // If the row is not empty because the hostname does not exist, put the ID in the list, else leave it set to zero.
  if( NULL != ( row = mysql_fetch_row( result ) ) )
  {
    id = strdup( row[0] );
  }

  if( debug )
  {
      printf("id =  %s\n", id);
  }

  mysql_close(conn);
  // return the hostid number
  return id;
}

void print_version( void )
{
  printf( "sqldelete version %s\n", VERSION );
}

/*
 *   Returns 1 if the supplied number is an integer, 0 if not
 */
int is_integer( char *number )
{
  long int n;
  if ( strspn( number, "-0123456789 " ) != strlen (number) )
    return(0);
  n = strtol( number, NULL, 10 );
  if( (errno != ERANGE) && (n >= INT_MIN) && (n <= INT_MAX) )
    return( 1 );
  return( 0 );
}

//display the list
void printList( struct node *head )
{
  //start from the beginning
  while(head != NULL)
  {
    printf("service->id = %s,service->secondhost = %s,service->hostgroup = %s\n",head->id, head->secondhost, head->hostgroup);
    head = head->next;
  }
}

//display the host list
void printhostList( struct hostnode *head )
{
  //start from the beginning
  while(head != NULL)
  {
    printf("host->hostname = %s host->id = %s\n",head->hostname, head->id );
    head = head->next;
  }
}

//display the host list
void getidForhostList( struct hostnode *head )
{
  if( debug )
    printf( "getidForhostList\n" );
  //start from the beginning
  while(head != NULL)
  {
    head->id = strdup( gethostname( head->hostname ) );
    head = head->next;
  }
}

//insert link at the first location
void insertFirst(char *id, struct node **head, bool initialize)
{
  //create a structure link
  struct node* link = (struct node*) malloc(sizeof(struct node));
  // allocate space for the id
  link->id  = malloc( sizeof (id) );
  //point it to old first node
  link->next = (*head);

  // put the ID of the opject in the list
  link->id = strdup( id );
  if( initialize )
  {
    link->secondhost = "n";
    link->hostgroup = "n";
  }

  if( debug )
    printf("Service link->id = %s\n", link->id );

	//point first to new first node
  (*head) = link;
}

//insert link at the first location
void insertFirsthost( char *hostname, struct hostnode **head )
{
  //create a structure link
  struct hostnode* link = (struct hostnode*) malloc(sizeof(struct hostnode));
  // allocate space for the hostname
  link->hostname  = malloc( sizeof (hostname) );

  //point it to old first node
  link->next = (*head);

  // put the hostname of the opject in the list
  link->hostname = strdup( hostname );
  link->id = "0";

  if( debug )
    printf("Host link->hostname = %s\n", link->hostname );

	//point first to new first node
  (*head) = link;
}

//find a struct with given key and update it
void updatestruct(char *id, int x, struct node *head)
{
   //start from the first link
   //struct node current = head;
   struct node *current = head;

   //if list is empty
   if(current->next == NULL)
   {
     printf( "Struct NULL\n" );
     return;
   }

   //navigate through list
   while(current->id != id) {

      //if it is last node
      if(current->next == NULL) {
         return;
      } else {
         //go to next link
         current = current->next;
      }
   }

}

unsigned long epoch_time( void )
{
  if( debug )
    fprintf(stdout, "%lu\n", (unsigned long)time(NULL));

  return (unsigned long)time(NULL);
}

int finish_with_error(MYSQL *con)
{
  fprintf(stderr, "%s\n", mysql_error(con));
  mysql_close(con);
  exit(1);
}

int mysqlquery(const char *query)
{
  if ( debug )
    printf( "query = %s\n", query );

  if( mysql_query( conn, query ) != 0 )
  {
    fprintf( stderr, "Query Failure\n" );
    return EXIT_FAILURE;
  }
}

// Delete the row in the MYSQL database by the id from the struct.
int mysqldeletelinklist( struct node *head, char *table )
{
int exitstatus = OK;
int i;
char *status = NULL;
char query[MAX_STRING] = {0};

  if( debug )
    printf( "Deleting %s\n", table );
  if ((conn = mysql_init(NULL)) == NULL)
  {
    fprintf(stderr, "Could not init DB\n");
    return EXIT_FAILURE;
  }

  if( mysql_real_connect(conn, mysqlhost, username, password, mysqldatabase, 0, NULL, 0) == NULL )
  {
    fprintf(stderr, "DB Connection Error\n");
    return EXIT_FAILURE;
  }

  //start from the beginning
  while(head != NULL)
  {
    if( head->secondhost == "n" && head->hostgroup == "n" )
    {
      snprintf(query, MAX_STRING, "DELETE FROM %s WHERE id = %s", table, head->id);
      mysqlquery(query);
    }
    else
    {
      // Need to go and walk through the Host List and if the Host is to be deleted, remove from the List.
      // start from the beginning
      while( host != NULL )
      {
        //printf( "Second Host\n");
        snprintf(query, MAX_STRING, "SELECT * FROM %s WHERE idSlave = %s and idMaster = %s;", "tbl_lnkServiceToHost", head->id, host->id );
        mysqlquery(query);
        MYSQL_RES *result = mysql_store_result(conn);

        if (result == NULL)
        {
            finish_with_error(conn);
        }

        int num_fields = mysql_num_fields(result);

        MYSQL_ROW row;

        //printf("service->id = %s, num_fields = %d\n",head->id, num_fields );

        while( NULL != ( row = mysql_fetch_row( result ) ) )
        {
          for(i = 0; i < num_fields; i++)
          {
            if( strncmp( row[i], "", 1 ) )
            {
              //printf( "Second Host loop\n");
              snprintf(query, MAX_STRING, "DELETE FROM %s WHERE idSlave = %s and idMaster = %s;", "tbl_lnkServiceToHost", head->id, host->id );
              mysqlquery(query);
            }
          }
        }

        host = host->next;
      }
    }
    head = head->next;
  }

  mysql_close(conn);
  return exitstatus;
}

// Delete the row in the MYSQL database by the id and the table.
int mysqldelete( char *table, char *field, char *id )
{
  int exitstatus = OK;
  char *status = NULL;
  char query[MAX_STRING] = {0};

  if ((conn = mysql_init(NULL)) == NULL)
  {
    fprintf(stderr, "Could not init DB\n");
    return EXIT_FAILURE;
  }

  if( mysql_real_connect(conn, mysqlhost, username, password, mysqldatabase, 0, NULL, 0) == NULL )
  {
    fprintf(stderr, "DB Connection Error\n");
    return EXIT_FAILURE;
  }

  snprintf(query, MAX_STRING, "DELETE FROM %s WHERE %s = %s", table, field, id);
  mysqlquery(query);

  mysql_close(conn);
  return exitstatus;
}

char *my_itoa(int num, char *str)
{
  if(str == NULL)
  {
    return NULL;
  }
  sprintf(str, "%d", num);
  return str;
}

/*
 * getservicename : Gets the ID for the services with the hostid from the MYSQL database
 *
 */
int getservicename( struct hostnode *head )
{
  int exitstatus = OK;
  int i;
  char *status = NULL;
  char query[MAX_STRING] = {0};

    if ((conn = mysql_init(NULL)) == NULL)
    {
      fprintf(stderr, "Could not init DB\n");
      return EXIT_FAILURE;
    }

    if( mysql_real_connect(conn, mysqlhost, username, password, mysqldatabase, 0, NULL, 0) == NULL )
    {
      fprintf(stderr, "DB Connection Error getservicename\n");
      return EXIT_FAILURE;
    }

  //start from the beginning
  while(head != NULL)
  {
    snprintf(query, MAX_STRING, "select idMaster from tbl_lnkServiceToHost WHERE idSlave = ('%s')", head->id);
    mysqlquery(query);


    MYSQL_RES *result = mysql_store_result(conn);

    if (result == NULL)
    {
        finish_with_error(conn);
    }

    int num_fields = mysql_num_fields(result);

    MYSQL_ROW row;

    while( NULL != ( row = mysql_fetch_row( result ) ) )
    {
      for(i = 0; i < num_fields; i++)
      {
        insertFirst(row[i] ? row[i] : "NULL", &service, true);        // Insert the data in to the linked struct
      }
    }
    head = head->next;
  }
  mysql_close(conn);
  return exitstatus;
}

/*
 * gethostinhostgroup : Gets the ID for the hostgroup with the hostid from the MYSQL database
 *
 */
int gethostinhostgroup( struct hostnode *head )
{
  int exitstatus = OK;
  int i;
  char *status = NULL;
  char query[MAX_STRING] = {0};


    if ((conn = mysql_init(NULL)) == NULL)
    {
      fprintf(stderr, "Could not init DB\n");
      return EXIT_FAILURE;
    }

    if( mysql_real_connect(conn, mysqlhost, username, password, mysqldatabase, 0, NULL, 0) == NULL )
    {
      fprintf(stderr, "DB Connection Error\n");
      return EXIT_FAILURE;
    }

  //start from the beginning
  while(head != NULL)
  {
    snprintf(query, MAX_STRING, "select idMaster from tbl_lnkHostgroupToHost WHERE idSlave = ('%s')", host->id);
    mysqlquery(query);


    MYSQL_RES *result = mysql_store_result(conn);

    if (result == NULL)
    {
        finish_with_error(conn);
    }

    int num_fields = mysql_num_fields(result);

    MYSQL_ROW row;


    while( NULL != ( row = mysql_fetch_row( result ) ) )
    {
      for(i = 0; i < num_fields; i++)
      {
        insertFirst(row[i] ? row[i] : "NULL", &hostgroup, true);        // Insert the data in to the linked struct
      }
    }
    head = head->next;
  }
  mysql_close(conn);
  return exitstatus;
}

//Get the hostgroup a service is in and update the linked list
int getserviceinhostgroup( struct node *head )
{
int exitstatus = OK;
int i;
char *status = NULL;
char query[MAX_STRING] = {0};

  if ((conn = mysql_init(NULL)) == NULL)
  {
    fprintf(stderr, "Could not init DB\n");
    return EXIT_FAILURE;
  }

  if( mysql_real_connect(conn, mysqlhost, username, password, mysqldatabase, 0, NULL, 0) == NULL )
  {
    fprintf(stderr, "DB Connection Error\n");
    return EXIT_FAILURE;
  }

  //start from the beginning
  while(head != NULL)
  {
    snprintf(query, MAX_STRING, "select idMaster from tbl_lnkServiceToHostgroup WHERE idMaster = ('%s')", head->id);
    mysqlquery(query);

    MYSQL_RES *result = mysql_store_result(conn);

    if (result == NULL)
    {
        finish_with_error(conn);
    }

    int num_fields = mysql_num_fields(result);

    MYSQL_ROW row;

    while( NULL != ( row = mysql_fetch_row( result ) ) )
    {
      for(i = 0; i < num_fields; i++)
      {
        head->hostgroup = "y"; // Set the service has a hostgroup to y
      }
    }

    head = head->next;
  }
  mysql_close(conn);
  return exitstatus;
}

//Get the hostgroup a service is in and update the linked list
int getserviceinsecondhost( struct node *head )
{
int exitstatus = OK;
int i;
char *status = NULL;
char query[MAX_STRING] = {0};

  if ((conn = mysql_init(NULL)) == NULL)
  {
    fprintf(stderr, "Could not init DB\n");
    return EXIT_FAILURE;
  }

  if( mysql_real_connect(conn, mysqlhost, username, password, mysqldatabase, 0, NULL, 0) == NULL )
  {
    fprintf(stderr, "DB Connection Error\n");
    return EXIT_FAILURE;
  }

  //start from the beginning
  while(head != NULL)
  {
    snprintf(query, MAX_STRING, "Select COUNT(*) FROM tbl_lnkServiceToHost WHERE idMaster = ('%s')", head->id);
    mysqlquery(query);

    MYSQL_RES *result = mysql_store_result(conn);

    if (result == NULL)
    {
        finish_with_error(conn);
    }

    int num_fields = mysql_num_fields(result);

    MYSQL_ROW row;

    while( NULL != ( row = mysql_fetch_row( result ) ) )
    {
      for(i = 0; i < num_fields; i++)
      {
        //printf("service->id = %s, count = %s\n", head->id, row[i] );
        if( atoi( row[i] ) > 1 )
          head->secondhost = "y"; // Set the service has a second host to y if the count is greater than 1
      }
    }

    head = head->next;
  }
  mysql_close(conn);
  return exitstatus;
}

int verify_nagios( void )
{
  int error_code;
  error_code = system( "/usr/local/nagios/bin/nagios -v /usr/local/nagios/etc/nagios.cfg >/dev/null");
  return error_code;
}

int write_configs( void )
{
  int error_code;

  if( debug )
    printf( "Writing Configs\n" );

  error_code = system( "/usr/bin/php -q /usr/local/nagiosxi/scripts/ccm_export.php >/dev/null");
  if( error_code != 0 )
  {
    fprintf(stderr, "Could not export nagios configs.\n");
    printf("Error = %i\n", error_code );
  }
  return error_code;
}

int revert_configs( void )
{
  int error_code;

  if( debug )
    printf( "Reverting Configs\n" );

  error_code = system( "/usr/local/nagiosxi/scripts/nom_restore_nagioscore_checkpoint.sh &>/dev/null");
  if( error_code != 0 )
  {
    fprintf(stderr, "Could not revert nagios configs.\n");
    printf("Error = %i\n", error_code );
  }
  return error_code;
}

int restart_nagios( void )
{
  int error_code;

  if( debug )
    printf( "Restart Nagios\n" );

  error_code = system( "/usr/local/nagiosxi/scripts/manage_services.sh restart nagios >/dev/null");
  if( error_code != 0 )
  {
    fprintf(stderr, "Could not restart nagios.\n");
    printf("Error = %i\n", error_code );
  }
  return error_code;
}

int create_snapshot( void )
{
  int error_code;

  if( debug )
    printf( "Create Snapshot\n" );

  error_code = system( "/usr/local/nagiosxi/scripts/nom_create_nagioscore_checkpoint.sh");
  if( error_code != 0 )
  {
    fprintf(stderr, "Could not create snapshot before deleting configurations.\n");
    printf("Error = %i\n", error_code );
  }
  return error_code;
}

void deleteall( struct hostnode *head )
{
  //Delete the Services
  mysqldeletelinklist( service, "tbl_service" );

  // Delete Host
  while( head != NULL )
  {
    // Delete Host from Host
    mysqldelete( "tbl_lnkHostToHost", "idMaster", head->id );

    // Delete Host from Host idSlave
    mysqldelete( "tbl_lnkHostToHost", "idSlave", head->id );

    // Delete Host from Hostgroups
    mysqldelete( "tbl_lnkHostgroupToHost", "idSlave", head->id );

    // Delete Host from Contacts
    mysqldelete( "tbl_lnkHostToContact", "idMaster", head->id );

    // Delete Host from ContactGroups
    mysqldelete( "tbl_lnkHostToContactgroup", "idMaster", head->id );

    // Delete Host from HostGroup
    mysqldelete( "tbl_lnkHostToHostgroup", "idMaster", head->id );

    // Delete Host from Hosttemplate
    mysqldelete( "tbl_lnkHostToHosttemplate", "idMaster", head->id );

    // Delete Host from ToVariabledefinition
    mysqldelete( "tbl_lnkHostToVariabledefinition", "idMaster", head->id );

    // Delete Depentant Host Hostdependency from Host
    mysqldelete( "tbl_lnkHostdependencyToHost_DH", "idSlave", head->id );

    // Delete Dependant Host Hostdependency from Host
    mysqldelete( "tbl_lnkHostdependencyToHost_H", "idSlave", head->id );

    // Delete Host Escalation from Host
    mysqldelete( "tbl_lnkHostescalationToHost", "idSlave", head->id );

    // Delete Service Dependency Dependant Host from Host
    mysqldelete( "tbl_lnkServicedependencyToHost_DH", "idSlave", head->id );

    // Delete Service Dependency Host from Host
    mysqldelete( "tbl_lnkServicedependencyToHost_H", "idSlave", head->id );

    // Delete Service Escalation Host
    mysqldelete( "tbl_lnkServiceescalationToHost", "idSlave", head->id );

    // Delete Service Groups
    mysqldelete( "tbl_lnkServicegroupToService", "idSlaveH", head->id );

    // Delete Service to Host links
    mysqldelete( "tbl_lnkServiceToHost", "idSlave", head->id );


    // Delete all Hosts
    mysqldelete( "tbl_host", "id", head->id );
    head = head->next;
  }
  check_empty_service( service, "tbl_lnkServiceToHost" );
}

int openfile( char *filename )
{
  if( debug )
  {
    printf( "openfile function\n" );
    printf( "The Host file name is %s\n", filename );
  }
  FILE *fpr;
  /*Char array to store string */
  char str[256];

  /*Opening the file in "r" mode*/
  fpr = fopen( filename, "r" );

  /*Error handling for file open*/
  if ( fpr == NULL )
  {
    puts( "Issue in opening the input file" );
    return( UNKNOWN );
  }

  /*Loop for reading the file till end*/
  while( 1 )
  {
    if(fgets( str, 255, fpr ) == NULL)
      break;
    else
    {
      // Remove the \n from the end of the string
      str[strcspn(str, "\n")] = 0;
      insertFirsthost( strdup( str ), &host );
      if( debug )
        printf( "str = %s\n", str );
    }
  }

  /*Closing the input file after reading*/
  fclose( fpr );
  return OK;
}


//https://www.geeksforgeeks.org/delete-occurrences-given-key-linked-list/
/* Given a reference (pointer to pointer) to the head of a list and
   a key, deletes all occurrence of the given key in linked list */
void deleteKey(struct hostnode **head_ref)
{
  // Store head node
  struct hostnode* temp = *head_ref, *prev;

  // If head node itself holds the key or multiple occurrences of key
  while(temp != NULL && ( strncmp( temp->id, "0", 1 ) == 0 ) )
  {
    *head_ref = temp->next;   // Changed head
    free(temp);               // free old head
    temp = *head_ref;         // Change Temp
    if ( (temp == NULL)  )
    {
      *head_ref = NULL;   // Changed head
      return;
    }
  }

  // Delete occurrences other than head
  while(temp != NULL)
  {
    // Search for the key to be deleted, keep track of the
    // previous node as we need to change 'prev->next'
    while(temp != NULL && ( strncmp( temp->id, "0", 1 ) != 0 ))
    {
      prev = temp;
      temp = temp->next;
    }

    // If key was not present in linked list
    if(temp == NULL)
    {
      printf( "NULL\n" );
      return;
    }

    // Unlink the node from linked list
    prev->next = temp->next;

    free(temp);  // Free memory

    //Update Temp for next iteration of outer loop
    temp = prev->next;
  }
}

// Check if the table is empty.
int check_empty_service( struct node *head, char *table )
{
int exitstatus = OK;
int i;
char *status = NULL;
char query[MAX_STRING] = {0};

  if( debug )
    printf( "Deleting Empty Service\n" );
  if ((conn = mysql_init(NULL)) == NULL)
  {
    fprintf(stderr, "Could not init DB\n");
    return EXIT_FAILURE;
  }

  if( mysql_real_connect(conn, mysqlhost, username, password, mysqldatabase, 0, NULL, 0) == NULL )
  {
    fprintf(stderr, "DB Connection Error\n");
    return EXIT_FAILURE;
  }

  //start from the beginning
  while(head != NULL)
  {

        //printf( "Second Host\n");
        //snprintf(query, MAX_STRING, "SELECT * FROM %s WHERE idMaster = %s;", "tbl_lnkServiceToHost", head->id );
        snprintf(query, MAX_STRING, "SELECT * FROM %s WHERE idMaster = %s;", table, head->id );
        mysqlquery(query);
        MYSQL_RES *result = mysql_store_result(conn);

        if (result == NULL)
        {
          printf( "Empty Host empty result\n");
          finish_with_error(conn);
        }

        int num_fields = mysql_num_fields(result);

        MYSQL_ROW row;

        //printf("service->id = %s, num_fields = %d\n",head->id, num_fields );

        if( NULL == ( row = mysql_fetch_row( result ) ) )
        {
          printf( "Empty Host loop\n");
          snprintf(query, MAX_STRING, "DELETE FROM %s WHERE id = %s;", "tbl_service", head->id );
          mysqlquery(query);
        }
    head = head->next;
  }

  mysql_close(conn);
  return exitstatus;
}